@@ -10,7 +10,8 @@
 # Copyright 2003-2009 by Paul McGuire
 #
 from pyparsing import Literal,CaselessLiteral,Word,Group,Optional,\
-    ZeroOrMore,Forward,nums,alphas,Regex
+    ZeroOrMore,Forward,nums,alphas,alphanums,Regex,ParseException,\
+    CaselessKeyword, Suppress
 import math
 import operator
 
@@ -19,10 +20,13 @@ exprStack = []
 def pushFirst( strg, loc, toks ):
     exprStack.append( toks[0] )
 def pushUMinus( strg, loc, toks ):
-    if toks and toks[0]=='-': 
+    for t in toks:
+      if t == '-': 
         exprStack.append( 'unary -' )
         #~ exprStack.append( '-1' )
         #~ exprStack.append( '*' )
+      else:
+        break
 
 bnf = None
 def BNF():
@@ -39,26 +43,25 @@ def BNF():
     global bnf
     if not bnf:
         point = Literal( "." )
-        e     = CaselessLiteral( "E" )
+        # use CaselessKeyword for e and pi, to avoid accidentally matching
+        # functions that start with 'e' or 'pi' (such as 'exp'); Keyword
+        # and CaselessKeyword only match whole words
+        e     = CaselessKeyword( "E" )
+        pi    = CaselessKeyword( "PI" )
         #~ fnumber = Combine( Word( "+-"+nums, nums ) + 
                            #~ Optional( point + Optional( Word( nums ) ) ) +
                            #~ Optional( e + Word( "+-"+nums, nums ) ) )
-        fnumber = Regex(r"[+-]?\d+(:?\.\d*)?(:?[eE][+-]?\d+)?")
-        ident = Word(alphas, alphas+nums+"_$")
+        fnumber = Regex(r"[+-]?\d+(?:\.\d*)?(?:[eE][+-]?\d+)?")
+        ident = Word(alphas, alphanums+"_$")
      
-        plus  = Literal( "+" )
-        minus = Literal( "-" )
-        mult  = Literal( "*" )
-        div   = Literal( "/" )
-        lpar  = Literal( "(" ).suppress()
-        rpar  = Literal( ")" ).suppress()
+        plus, minus, mult, div = map(Literal, "+-*/")
+        lpar, rpar = map(Suppress, "()")
         addop  = plus | minus
         multop = mult | div
         expop = Literal( "^" )
-        pi    = CaselessLiteral( "PI" )
         
         expr = Forward()
-        atom = (Optional("-") + ( pi | e | fnumber | ident + lpar + expr + rpar ).setParseAction( pushFirst ) | 
+        atom = ((0,None)*minus + ( pi | e | fnumber | ident + lpar + expr + rpar | ident ).setParseAction( pushFirst ) | 
                 Group( lpar + expr + rpar )).setParseAction(pushUMinus) 
         
         # by defining exponentiation as "atom [ ^ factor ]..." instead of "atom [ ^ atom ]...", we get right-to-left exponents, instead of left-to-righ
@@ -81,10 +84,11 @@ opn = { "+" : operator.add,
 fn  = { "sin" : math.sin,
         "cos" : math.cos,
         "tan" : math.tan,
+        "exp" : math.exp,
         "abs" : abs,
         "trunc" : lambda a: int(a),
         "round" : round,
-        "sgn" : lambda a: abs(a)>epsilon and cmp(a,0) or 0}
+        "sgn" : lambda a: (a > epsilon) - (a < -epsilon) }
 def evaluateStack( s ):
     op = s.pop()
     if op == 'unary -':
@@ -100,7 +104,7 @@ def evaluateStack( s ):
     elif op in fn:
         return fn[op]( evaluateStack( s ) )
     elif op[0].isalpha():
-        return 0
+        raise Exception("invalid identifier '%s'" % op)
     else:
         return float( op )
 
@@ -108,13 +112,19 @@ if __name__ == "__main__":
     
     def test( s, expVal ):
         global exprStack
-        exprStack = []
-        results = BNF().parseString( s )
-        val = evaluateStack( exprStack[:] )
-        if val == expVal:
-            print s, "=", val, results, "=>", exprStack
+        exprStack[:] = []
+        try:
+            results = BNF().parseString( s, parseAll=True )
+            val = evaluateStack( exprStack[:] )
+        except ParseException as pe:
+            print(s, "failed parse:", str(pe))
+        except Exception as e:
+            print(s, "failed eval:", str(e))
         else:
-            print s+"!!!", val, "!=", expVal, results, "=>", exprStack
+            if val == expVal:
+                print(s, "=", val, results, "=>", exprStack)
+            else:
+                print(s+"!!!", val, "!=", expVal, results, "=>", exprStack)
   
     test( "9", 9 )
     test( "-9", -9 )
@@ -140,12 +150,15 @@ if __name__ == "__main__":
     test( "round(E)", round(math.e) )
     test( "round(-E)", round(-math.e) )
     test( "E^PI", math.e**math.pi )
+    test( "exp(0)", 1 )
+    test( "exp(1)", math.e )
     test( "2^3^2", 2**3**2 )
     test( "2^3+2", 2**3+2 )
     test( "2^3+5", 2**3+5 )
     test( "2^9", 2**9 )
     test( "sgn(-2)", -1 )
     test( "sgn(0)", 0 )
+    test( "foo(0.1)", None )
     test( "sgn(0.1)", 1 )
 
 
@@ -176,4 +189,4 @@ sgn(-2) = -1 ['sgn', '-2'] => ['-2', 'sgn']
 sgn(0) = 0 ['sgn', '0'] => ['0', 'sgn']
 sgn(0.1) = 1 ['sgn', '0.1'] => ['0.1', 'sgn']
 >Exit code: 0
-"""
\ No newline at end of file
+"""
