@@ -28,7 +28,7 @@ for toks,strt,end in link.scanString(htmlText):
 # Create dictionary from list comprehension, assembled from each pair of tokens returned 
 # from a matched URL.
 pprint.pprint( 
-    dict( [ (toks.body,toks.startA.href) for toks,strt,end in link.scanString(htmlText) ] )
+    {  toks.body:toks.startA.href for toks,strt,end in link.scanString(htmlText)  }
     )
 
 
